"""
Pydantic Models for MONITOR - system/config/restore

Runtime validation models for system/config/restore configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class RestoreModel(BaseModel):
    """
    Pydantic model for system/config/restore configuration.
    
    Restore system configuration from uploaded file or from USB.
    
    Validation Rules:        - source: pattern=        - usb_filename: pattern=        - config_id: pattern=        - password: pattern=        - scope: pattern=        - vdom: pattern=        - confirm_password_mask: pattern=        - file_content: pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    source: str = Field()    
    usb_filename: str | None = Field(default=None)    
    config_id: Any = Field(default=None)    
    password: str | None = Field(default=None)    
    scope: str = Field()    
    vdom: str | None = Field(default=None)    
    confirm_password_mask: Any = Field(default=None)    
    file_content: str | None = Field(default=None)    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "RestoreModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "RestoreModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T02:10:52.115317Z
# ============================================================================